@@ -7,7 +7,6 @@ import uuid
 
 from django.db import models
 from django.conf import settings
-from unixtimestampfield.fields import UnixTimeStampField
 
 User = settings.AUTH_USER_MODEL
 Group = settings.GROUP_MODEL
@@ -39,7 +38,7 @@ class SignedPreKey(models.Model):
     cipher = models.CharField(max_length=88)
     salt = models.CharField(max_length=44)
     active = models.BooleanField(default=False)
-    timestamp = UnixTimeStampField(auto_now_add=True)
+    # timestamp = models.DateTimeField(auto_now_add=True)
 
     class Meta:
         constraints = [models.UniqueConstraint(fields=['keyId', 'user'], name='unique_signed_prekey')]
