@@ -10,22 +10,22 @@ from django.db.models import Q
 from firebase_admin import db
 from django.conf import settings
 
-
 DEFAULT_APP = settings.DEFAULT_APP
 FIREBASE_REF = settings.FIREBASE_REF
 FIREBASE_REF_DEV = settings.FIREBASE_REF_DEV
 
+
 #
 # @author Omar Basem
 #
 
 class StickProtocol():
 
-    def __init__(self, UserModel, GroupModel):
+    def __init__(self, UserModel, DeviceModel, GroupModel):
         self.User = UserModel
+        self.Device = DeviceModel
         self.Group = GroupModel
 
-
     def process_pre_key_bundle(self, data, user):
         """
         A user must upload their PreKeyBundle at registration time. Before uploading their PreKeyBundle, they need to verify
@@ -43,15 +43,14 @@ class StickProtocol():
             PreKey.objects.create(public=preKey['public'], keyId=preKey["id"], user=user, cipher=preKey['cipher'],
                                   salt=preKey['salt'])
         user.passwordSalt = data["passwordSalt"]
-        user.set_password(data["passwordHash"]) # This will create a "Double-Hashed" password
+        user.set_password(data["passwordHash"])  # This will create a "Double-Hashed" password
         user.oneTimeId = data["oneTimeId"]
         user.nextPreKeyId = data['nextPreKeyId']
         user.finishedRegistration = True
         user.save()
+        self.Device.objects.create(user=user, deviceId=data['deviceId'], name=data['deviceName'])
         Party.objects.create(user=user)
 
-
-
     def process_pre_keys(self, data, user):
         """
         A user would need to refill their PreKeys on the server every while whenever it goes below a certain N value.
@@ -64,8 +63,6 @@ class StickProtocol():
         user.nextPreKeyId = data['nextPreKeyId']
         user.save()
 
-
-
     def get_pre_key_bundle(self, data):
         """
         The following get method is used to fetch the PreKeyBundle of user to create a pairwise signal session. The request must contain
@@ -100,9 +97,6 @@ class StickProtocol():
             preKey.save()
         return PKB
 
-
-
-
     def get_pre_key_bundles(self, currentUser, users_id):
         """
         Similar to the above method, but fetches PreKeyBundles of several users at once. This method allows a user to
@@ -146,8 +140,6 @@ class StickProtocol():
         dict = {"bundles": bundles, "users_id": users_id}
         return dict
 
-
-
     def get_sender_key(self, data, user):
         """
         This method is used to fetch the SenderKey of a stickySession.
@@ -167,19 +159,20 @@ class StickProtocol():
 
         # You need to check whether the user is authorized to fetch that SenderKey
         authorized = False
-        if user in member.blocked.all(): # A blocked user is not authorized
+        if user in member.blocked.all():  # A blocked user is not authorized
             return {'authorized': authorized}
-        if isInvitation: # An invited user is authorized
+        if isInvitation:  # An invited user is authorized
             group = self.Group.objects.get(id=stickId[:36])
             if group in user.invited_groups.all():
                 authorized = True
-        elif stickId.startswith(user.party.id): # A user is authorized to fetch SenderKeys of their own profile (user.party.id)
+        elif stickId.startswith(
+                user.party.id):  # A user is authorized to fetch SenderKeys of their own profile (user.party.id)
             authorized = True
         else:
             groupId = stickId[:36] if isSticky else data['groupId']
             group = self.Group.objects.filter(id=groupId).first()
             if group:
-                if group in user.groups.all(): # A group member is authorized
+                if group in user.groups.all():  # A group member is authorized
                     authorized = True
             else:
                 party = Party.objects.get(id=stickId[:36])
@@ -187,31 +180,31 @@ class StickProtocol():
                 if party.user and (user in party.user.connections.all() or user.phone in party.user.contacts):
                     authorized = True
                 else:
-                    if user in party.connections.all(): # A user in the connections list of a party should be authorized
+                    if user in party.connections.all():  # A user in the connections list of a party should be authorized
                         authorized = True
                     else:
                         userGroups = user.groups.all()
                         for group in party.groups.all():
-                            if group in userGroups: # If a Party and a User have a mutual Group, then that user is authorized
+                            if group in userGroups:  # If a Party and a User have a mutual Group, then that user is authorized
                                 authorized = True
                                 break
-        if not authorized: # if NOT authorized, return 401
+        if not authorized:  # if NOT authorized, return 401
             return {'authorized': authorized}
 
-        if isSticky: # Fetching the SenderKey of a sticky session
-            if memberId != user.id: # Trying to fetch DSK
+        if isSticky:  # Fetching the SenderKey of a sticky session
+            if memberId != user.id:  # Trying to fetch DSK
                 senderKey = DecryptingSenderKey.objects.filter(stickId=stickId, ofUser=memberId,
-                                                           forUser=user).first()
-            else: # Trying to fetch ESK
+                                                               forUser=user).first()
+            else:  # Trying to fetch ESK
                 partyId = stickId[:36]
                 chainId = stickId[36:]
                 senderKey = EncryptingSenderKey.objects.filter(partyId=partyId, chainId=chainId, user=user).first()
-        else: # Trying to fetch DSK of a standard group session
+        else:  # Trying to fetch DSK of a standard group session
             oneTimeId = data['oneTimeId']
             senderKey = DecryptingSenderKey.objects.filter(stickId=stickId, ofOneTimeId=oneTimeId,
                                                            forOneTimeId=user.oneTimeId).first()
         key = None
-        if senderKey: # If the SenderKey exists, we will return it
+        if senderKey:  # If the SenderKey exists, we will return it
             if memberId != user.id:
                 key = senderKey.key
             else:
@@ -230,8 +223,6 @@ class StickProtocol():
             ref.update({stickId + '--' + str(user.id): user.phone})
         return {'authorized': authorized, 'senderKey': key}
 
-
-
     def get_standard_sender_keys(self, data, user, group):
         """
         This method is used to fetch the standard session sender keys of a group.
@@ -250,8 +241,6 @@ class StickProtocol():
             senderKeys[id] = key
         return {'authorized': True, 'senderKeys': senderKeys}
 
-
-
     def get_uploaded_sender_keys(self, data, user):
         """
         Before a user makes an upload they need to know which stickId to use (whether the current sticky session has expired).
@@ -271,14 +260,14 @@ class StickProtocol():
             isProfile = data['isProfile']
         membersIds = []
         if not isProfile and 'partyId' not in data:
-            if len(groups_ids) == 1 and len(connections_ids) == 0: # Sharing with a single group
-                if isSticky: # Using sticky session
+            if len(groups_ids) == 1 and len(connections_ids) == 0:  # Sharing with a single group
+                if isSticky:  # Using sticky session
                     membersIds = self.Group.objects.get(id=groups_ids[0]).get_members_ids()
                     partyId = groups_ids[0]
-                else: # Using standard session
+                else:  # Using standard session
                     membersIds = self.Group.objects.get(id=groups_ids[0]).get_members_otids()
                     partyId = data["stickId"]
-            else: # Sharing with a collection of groups and/or users
+            else:  # Sharing with a collection of groups and/or users
                 if len(groups_ids) > 0 and len(connections_ids) > 0:
                     party = Party.objects.filter(Q(groups=groups_ids) & Q(connections=connections_ids)).first()
                 elif len(groups_ids) > 0:
@@ -286,7 +275,7 @@ class StickProtocol():
                 else:
                     connections_ids.append(user.id)
                     party = Party.objects.filter(Q(groups=None) & Q(connections=connections_ids)).first()
-                if party == None: # Create a new Party object if does not exist
+                if party == None:  # Create a new Party object if does not exist
                     party = Party.objects.create()
                     party.groups.set(groups_ids)
                     party.connections.set(connections_ids)
@@ -301,14 +290,14 @@ class StickProtocol():
                 if user.id not in membersIds:
                     membersIds.append(user.id)
                 partyId = party.id
-        elif 'partyId' in data: # Sharing to the party of a user
+        elif 'partyId' in data:  # Sharing to the party of a user
             membersIds = [user.id]
             if not (len(groups_ids) == 1 and len(connections_ids) == 0):
                 party = Party.objects.get(id=data['partyId'])
                 if party.user and party.user.id != user.id:
                     membersIds.append(party.user.id)
             partyId = data['partyId']
-        else: # Sharing to the currentUser's party (currentUsers' profile)
+        else:  # Sharing to the currentUser's party (currentUsers' profile)
             membersIds = connections_ids
             partyId = Party.objects.get(user=user).id
         dict = {}
@@ -320,13 +309,13 @@ class StickProtocol():
         senderKeys = EncryptingSenderKey.objects.filter(partyId=partyId, user=user).order_by('-chainId')
         if len(senderKeys) > 0:
             activeSenderKey = senderKeys[0]
-            if not isSticky: # A standard session is valid
+            if not isSticky:  # A standard session is valid
                 dict[user.id] = {'exists': True}
-            elif activeSenderKey.step < 100: # Check whether is sticky session has not expired
+            elif activeSenderKey.step < 100:  # Check whether is sticky session has not expired
                 chainId = activeSenderKey.chainId
                 dict[user.id] = {'exists': True}
                 responseDict["step"] = activeSenderKey.step
-            else: # Sticky session has expired, increment chainId by 1
+            else:  # Sticky session has expired, increment chainId by 1
                 chainId = int(activeSenderKey.chainId) + 1
                 dict[user.id] = {'exists': False}
                 bundlesToFetch.append(user.id)
@@ -336,7 +325,7 @@ class StickProtocol():
         if not isSticky:
             stickId = partyId
 
-        for memberId in membersIds: # loop of the target users and check if they have their SenderKey
+        for memberId in membersIds:  # loop of the target users and check if they have their SenderKey
             if isSticky:
                 if user.id != memberId:
                     senderKey = DecryptingSenderKey.objects.filter(stickId=stickId, ofUser=user,
@@ -362,8 +351,6 @@ class StickProtocol():
         responseDict["bundlesToFetch"] = bundlesToFetch
         return responseDict
 
-
-
     def get_active_stick_id(self, data, currentUser):
         """
         This method gets the active sticky session stickId associated with a particular partyId that already exists, and
@@ -381,7 +368,6 @@ class StickProtocol():
         responseDict['stickId'] = str(partyId) + str(chainId)
         return responseDict
 
-
     def process_sender_key(self, data, user):
         """
         This method is used to save a SenderKey of a sticky session for a user. Typically used when a user
@@ -395,8 +381,6 @@ class StickProtocol():
         decryptingSenderKey.forUser = forUser
         decryptingSenderKey.save()
 
-
-
     def process_sender_keys(self, data, user):
         """
         This method is used to save SenderKeys of multiple users at once. Before making an upload, and after
@@ -408,12 +392,12 @@ class StickProtocol():
         for id in users_id:
             data = keys[id]
             preKey = PreKey.objects.get(keyId=data['preKeyId'], user__id=id)
-            if id != user.id: # Other user? Create a DSK
+            if id != user.id:  # Other user? Create a DSK
                 forUser = self.User.objects.get(id=data['forUser'])
                 DecryptingSenderKey.objects.create(key=data['key'],
                                                    stickId=data['stickId'], ofUser=user,
                                                    preKey=preKey, forUser=forUser)
-            else: # Current user? Create an ESK
+            else:  # Current user? Create an ESK
                 partyId = data['stickId'][:36]
                 chainId = data['stickId'][36:]
                 EncryptingSenderKey.objects.create(keyId=data['id'], preKey=preKey,
@@ -424,7 +408,6 @@ class StickProtocol():
         if "group_id" in data:
             user.just_added_groups.remove(data["group_id"])
 
-
     def process_standard_sender_keys(self, data, user):
         """
         This method is used to upload the SenderKeys of a standard session.
@@ -435,7 +418,6 @@ class StickProtocol():
             DecryptingSenderKey.objects.create(key=senderKey, stickId=stickId, ofUser=user,
                                                forOneTimeId=oneTimeId, ofOneTimeId=user.oneTimeId)
 
-
     def update_active_spk(self, data, user):
         """
         This method is used to update the active signed prekey for a user
@@ -446,8 +428,6 @@ class StickProtocol():
         SignedPreKey.objects.create(user=user, public=data['public'], signature=data["signature"],
                                     keyId=data['id'], cipher=data['cipher'], salt=data['salt'], active=True)
 
-
-
     def verify_password_and_get_keys(self, data, user):
         """
         This Login method should be called after the user have verified their phone number and got their LimitedAccessToken.
@@ -462,7 +442,7 @@ class StickProtocol():
         as well as any of the DSKs the was sent to them, which they can fetch again from the server as needed.
         """
         # user = self.User.objects.get(phone=data['phone'])
-        if user.check_password(data['passwordHash']): # This will create a "double-hash" and verify it
+        if user.check_password(data['passwordHash']):  # This will create a "double-hash" and verify it
             identityKey = IdentityKey.objects.get(user=user)
             signedPreKeysList = SignedPreKey.objects.filter(user=user)
             preKeysList = PreKey.objects.filter(user=user)
@@ -477,7 +457,8 @@ class StickProtocol():
             }
             signedPreKeys = []
             for spk in signedPreKeysList:
-                key = {'id': spk.keyId, 'public': spk.public, 'cipher': spk.cipher, 'salt': spk.salt, 'signature': spk.signature, 'active': spk.active}
+                key = {'id': spk.keyId, 'public': spk.public, 'cipher': spk.cipher, 'salt': spk.salt,
+                       'signature': spk.signature, 'active': spk.active, 'timestamp': spk.timestamp}
                 signedPreKeys.append(key)
             bundle['signedPreKeys'] = signedPreKeys
             preKeys = []
@@ -494,6 +475,8 @@ class StickProtocol():
             bundle['senderKeys'] = senderKeys
             user.oneTimeId = uuid.uuid4()
             user.save()
+            if not self.Device.objects.filter(user=user, deviceId=data['deviceId']).exists():
+                self.Device.objects.create(user=user, deviceId=data['deviceId'], name=data['deviceName'])
             return {"bundle": bundle, "verify": True}
         else:
             return {"verify": False}
